#ifndef SYSTEM_MOUSE_H
#define SYSTEM_MOUSE_H

#include "math/vec.h"
#include "system/defs.h"	//needed for keystate interpretation
#include "system/display.h"

//exposed

class PMouseHandler {
public:
	/*
	you know quake1 was on to something when they lumped all keyboard, mouse, mwheel, joystick, etc button's press/releases into one place...
	maybe I should do that too, and just keep track of mouse positions here ...
	make up my "register buttons with the system" interface ...
	*/
	enum Button_t {
		BUTTON_LEFT,
		BUTTON_RIGHT,
		BUTTON_MIDDLE,
		MAX_BUTTON
	};

	virtual void update() = 0;
	virtual void setButtonPress(Button_t button, bool down) = 0;
	virtual void setCursorPos(int x, int y) = 0;
	virtual void accumDZ(short addDZ) = 0;
	virtual int getDZ() const = 0;
	
	virtual const vec2i &getPos() const = 0;
	virtual const vec2i &getMove() const = 0;
	virtual const vec2f &getFPos() const = 0;
	virtual const vec2f &getFMove() const = 0;
	virtual const char getButtonState(int button) const = 0;
};

class IMouseHandler : public PMouseHandler {
	PMouseHandler *imp;
public:
	IMouseHandler();
	void setImp(PMouseHandler *_imp);
	virtual void update();
	virtual void setButtonPress(Button_t button, bool down);
	virtual void setCursorPos(int x, int y);
	virtual void accumDZ(short addDZ);
	virtual int getDZ() const;
	virtual const vec2i &getPos() const;
	virtual const vec2i &getMove() const;
	virtual const vec2f &getFPos() const;
	virtual const vec2f &getFMove() const;
	virtual const char getButtonState(int button) const;
};

//internal

class MouseHandler : public PMouseHandler {
public:
	MouseHandler() {reset();}
	void reset();

	//call once per frame to update all mouse stats
	virtual void update();

	//variable updates
	virtual void setButtonPress(Button_t button, bool down);
	virtual void setCursorPos(int x, int y);
	virtual void accumDZ(short addDZ);
	virtual int getDZ() const { return dz; }

	virtual const vec2i &getPos() const { return pos; }
	virtual const vec2i &getMove() const { return move; }
	virtual const vec2f &getFPos() const { return posf; }
	virtual const vec2f &getFMove() const { return movef; }
	virtual const char getButtonState(int button) const { return buttonState[button]; }

protected:

	//and all the stats...
	char buttonState[MAX_BUTTON];

	//movement in pixels
	vec2i pos;	//int x,y
	vec2i move;	//int dx,dy

	//movement in screen fractions
	//float xf, yf, dxf, dyf;
	vec2f posf;
	vec2f movef;

	//scroll wheel movement
	int dz, dzLast;

	//the latest mouse update position - taken into account upon 'update' call
	vec2i newpos;	//int newX, newY;

	//right now we're using glut calls for display window sizes
	//swap that with a display object pointer (?)

	//dont let the public see these =) allow people only button state variable access
	char buttonPress[MAX_BUTTON];
	char buttonPressLast[MAX_BUTTON];
};

#endif	//SYSTEM_MOUSE_H